"""
Contrib packages are stated as less stable as the core itself. This way we can still develop and make PyPlanet better
on the long term without breaking any core code.

In the contrib package you can find many useful packages, such as the command manager and parameter parser.
"""


class CoreContrib:
	async def on_start(self):
		pass
